Reworking for new prime-search system. Add function for working out how
authormdw <mdw>
Wed, 22 Dec 1999 15:50:29 +0000 (15:50 +0000)
committermdw <mdw>
Wed, 22 Dec 1999 15:50:29 +0000 (15:50 +0000)
many iterations to use for a particular number.

rabin.c
rabin.h

diff --git a/rabin.h b/rabin.h
index a161fa9..fca099d 100644 (file)
--- a/rabin.h
+++ b/rabin.h
@@ -1,6 +1,6 @@
 /* -*-c-*-
  *
- * $Id: rabin.h,v 1.2 1999/12/10 23:29:48 mdw Exp $
+ * $Id: rabin.h,v 1.3 1999/12/22 15:50:29 mdw Exp $
  *
  * Miller-Rabin primality test
  *
 /*----- Revision history --------------------------------------------------* 
  *
  * $Log: rabin.h,v $
+ * Revision 1.3  1999/12/22 15:50:29  mdw
+ * Reworking for new prime-search system.  Add function for working out how
+ * many iterations to use for a particular number.
+ *
  * Revision 1.2  1999/12/10 23:29:48  mdw
  * Change header file guard names.
  *
@@ -55,8 +59,8 @@
 #  include "mpmont.h"
 #endif
 
-#ifndef CATACOMB_PGEN_H
-#  include "pgen.h"
+#ifndef CATACOMB_PFILT_H
+#  include "pfilt.h"
 #endif
 
 /*----- Data structures ---------------------------------------------------*/
@@ -100,7 +104,7 @@ extern void rabin_destroy(rabin */*r*/);
  * Arguments:  @rabin *r@ = pointer to Rabin-Miller context
  *             @mp *g@ = base to test the number against
  *
- * Returns:    Either @PGEN_COMPOSITE@ if the test failed, or @PGEN_MAYBE@
+ * Returns:    Either @PGEN_FAIL@ if the test failed, or @PGEN_TRY@
  *             if it succeeded.
  *
  * Use:                Performs a single iteration of the Rabin-Miller primality
@@ -109,6 +113,18 @@ extern void rabin_destroy(rabin */*r*/);
 
 extern int rabin_test(rabin */*r*/, mp */*g*/);
 
+/* --- @rabin_iters@ --- *
+ *
+ * Arguments:  @unsigned len@ = number of bits in value
+ *
+ * Returns:    Number of iterations recommended.
+ *
+ * Use:                Returns the recommended number of iterations to ensure that a
+ *             number with @len@ bits is really prime.
+ */
+
+extern int rabin_iters(unsigned /*len*/);
+
 /*----- That's all, folks -------------------------------------------------*/
 
 #ifdef __cplusplus
diff --git a/rabin.c b/rabin.c
index 7a7e234..58d5291 100644 (file)
--- a/rabin.c
+++ b/rabin.c
@@ -1,6 +1,6 @@
 /* -*-c-*-
  *
- * $Id: rabin.c,v 1.2 1999/12/10 23:29:48 mdw Exp $
+ * $Id: rabin.c,v 1.3 1999/12/22 15:50:29 mdw Exp $
  *
  * Miller-Rabin primality test
  *
 /*----- Revision history --------------------------------------------------* 
  *
  * $Log: rabin.c,v $
+ * Revision 1.3  1999/12/22 15:50:29  mdw
+ * Reworking for new prime-search system.  Add function for working out how
+ * many iterations to use for a particular number.
+ *
  * Revision 1.2  1999/12/10 23:29:48  mdw
  * Change header file guard names.
  *
@@ -41,6 +45,7 @@
 /*----- Header files ------------------------------------------------------*/
 
 #include "mp.h"
+#include "mpbarrett.h"
 #include "mpmont.h"
 #include "pgen.h"
 #include "rabin.h"
@@ -79,7 +84,7 @@ void rabin_create(rabin *r, mp *m)
 
   /* --- Compute %$(m - 1)R \bmod m$% --- */
 
-  r->m1 = mpmont_mul(&r->mm, MP_NEW, m1, r->mm.r2);
+  r->m1 = mp_sub(MP_NEW, m, r->mm.r);
   mp_drop(m1);
 }
 
@@ -105,7 +110,7 @@ void rabin_destroy(rabin *r)
  * Arguments:  @rabin *r@ = pointer to Rabin-Miller context
  *             @mp *g@ = base to test the number against
  *
* Returns:    Either @PGEN_COMPOSITE@ if the test failed, or @PGEN_MAYBE@
  *             if it succeeded.
  *
  * Use:                Performs a single iteration of the Rabin-Miller primality
@@ -117,7 +122,7 @@ int rabin_test(rabin *r, mp *g)
   mp *y;
   mp *dd, *spare = MP_NEW;
   size_t j;
-  int rc = PGEN_COMPOSITE;
+  int rc = PGEN_FAIL;
 
   /* --- Calculate %$y R = g^r R \bmod m$% --- *
    *
@@ -127,7 +132,7 @@ int rabin_test(rabin *r, mp *g)
 
   y = mpmont_expr(&r->mm, MP_NEW, g, r->r);
   if (MP_CMP(y, ==, r->mm.r) || MP_CMP(y, ==, r->m1)) {
-    rc = PGEN_MAYBE;
+    rc = PGEN_PASS;
     goto done;
   }
 
@@ -138,11 +143,13 @@ int rabin_test(rabin *r, mp *g)
    */
 
   for (j = 1; j < r->s; j++) {
-    dd = mpmont_mul(&r->mm, spare, y, y); spare = y; y = dd;
+    dd = mp_sqr(spare, y);
+    dd = mpmont_reduce(&r->mm, dd, dd);
+    spare = y; y = dd;
     if (MP_CMP(y, ==, r->mm.r))
       break;
     if (MP_CMP(y, ==, r->m1)) {
-      rc = PGEN_MAYBE;
+      rc = PGEN_PASS;
       break;
     }
   }
@@ -156,4 +163,54 @@ done:
   return (rc);
 }
 
+/* --- @rabin_iters@ --- *
+ *
+ * Arguments:  @unsigned len@ = number of bits in value
+ *
+ * Returns:    Number of iterations recommended.
+ *
+ * Use:                Returns the recommended number of iterations to ensure that a
+ *             number with @len@ bits is really prime.
+ */
+
+int rabin_iters(unsigned len)
+{
+  static struct {
+    unsigned b;
+    int i;
+  } *p, *q, tab[] = {
+    { 100, 27 },
+    { 150, 18 },
+    { 200, 15 },
+    { 250, 12 },
+    { 300, 9 },
+    { 350, 8 },
+    { 400, 7 },
+    { 450, 6 },
+    { 550, 5 },
+    { 650, 4 },
+    { 850, 3 },
+    { 1300, 2 }
+  };
+
+  unsigned i;
+
+  /* --- Binary search through the table --- */
+
+  p = tab;
+  q = tab + (sizeof(tab)/sizeof(tab[0]));
+  for (;;) {
+    i = (q - p) / 2;
+    if (!i)
+      break;
+    if (len >= p[i].b && len < p[i + 1].b)
+      break;
+    if (len > p[i].b)
+      p = p + i;
+    else
+      q = p + i;
+  }
+  return (p[i].i);
+}
+
 /*----- That's all, folks -------------------------------------------------*/
@@ -105,7 +110,7 @@ void rabin_destroy(rabin *r)
  *             @mp *g@ = base to test the number against
  *
- * Returns:    Either @PGEN_COMPOSITE@ if the test failed, or @PGEN_MAYBE@
+ * Returns:    Either @PGEN_FAIL@ if the test failed, or @PGEN_PASS@
  *             if it succeeded.
  *